<?php
// $Id: xc_metadata.delete.inc,v 1.1.2.5 2010/02/10 23:41:41 pkiraly Exp $

/**
 * Metadata deletion form
 */
function xc_metadata_delete_form(&$form_state) {
  $form['instructions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Instructions'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );

  $form['instructions']['how'] = array(
    '#type' => 'item',
    '#title' => t('How it works'),
    '#value' => t('This batch process <strong>completely deletes and removes</strong>
                   all entity and node from Drupal based on the selected criteria. ')
  );

  $form['types'] = array(
    '#type' => 'fieldset',
    '#title' => t('Types'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE
  );

  $type_options = array('all' => t('<em>Delete all types</em>'));
  foreach (xc_metadata_get_entities() as $key => $value) {
    $type_options[$key] = empty($value['title']) ? $key : $value['title'];
  }
  $form['types']['types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Entity types to delete'),
    '#options' => $type_options,
    '#default_value' => array('all'),
    '#required' => TRUE
  );

  $form['locations'] = array(
    '#type' => 'fieldset',
    '#title' => t('Storage locations'),
    '#description' => t('This option only works for enities with explicit '
      . 'locations declared, such as entities imported with Import/Export '
      . 'module. It does not work for entities for with its locations defined '
      . 'by its source, such as those imported by the OAI Harvester.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE
  );

  $location_options = array('all' => t('<em>Delete from all locations</em>'));
  foreach (xc_location_get_all() as $key => $value) {
    $location_options[$key] = empty($value->name) ? 'Location #' . $key
      : $value->name;
  }
  $form['locations']['locations'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Storage locations of entities to delete'),
    '#options' => $location_options,
    '#default_value' => array('all'),
    '#required' => TRUE
  );

  $form['created'] = array(
    '#type' => 'fieldset',
    '#title' => t('Date created'),
    '#description' => t('Range dates entities to delete were created. Leave '
      . 'blank for all dates.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );

  $form['created']['from'] = array(
    '#type' => 'date',
    '#title' => t('From'),
    '#default_value' => array('year' => 1970, 'month' => 1, 'day' => 1),
    '#required' => TRUE
  );

  $form['created']['to'] = array(
    '#type' => 'date',
    '#title' => t('To'),
    '#required' => TRUE
  );

  $form['updated'] = array(
    '#type' => 'fieldset',
    '#title' => t('Date modified or updated'),
    '#description' => t('Range dates entities to delete were last modified or '
      . 'updated. Leave blank for all dates.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );

  $form['updated']['from'] = array(
    '#type' => 'date',
    '#title' => t('From'),
    '#default_value' => array('year' => 1970, 'month' => 1, 'day' => 1),
    '#required' => TRUE
  );

  $form['updated']['to'] = array(
    '#type' => 'date',
    '#title' => t('To'),
    '#required' => TRUE
  );

  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE
  );

  $form['advanced']['batch_limit'] = array(
    '#type' => 'select',
    '#title' => t('Number of entity/nodes processed in a batch'),
    '#description' => t('Set appropriately to what your system resources can
                         handle.'),
    '#multiple' => FALSE,
    '#options' => array(5 => 5, 25 => 25, 50 => 50, 100 => 100, 200 => 200, 
      500 => 500, 1000 => 1000, 2000 => 2000, 5000 => 5000),
    '#default_value' => 200,
    '#required' => TRUE
  );

  $form['confirm'] = array(
    '#type' => 'value',
    '#value' => FALSE
  );

  $form['warning'] = array(
    '#type' => 'item',
    '#prefix' => '<div class="messages error">',
    '#suffix' => '</div>',
    '#value' => t('<center>Are you sure you want to delete all metadata entities '
      . 'and nodes that meet the specified settings? <br />'
      . '<strong>This process may destroy a lot of stored metadata and cannot '
      . 'be undone!</strong></center>')
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete entities/nodes')
  );

  return $form;
}

/**
 * Metadata deletion form submission
 */
function xc_metadata_delete_form_submit($form, &$form_state) {
  // Types
  if (isset($form_state['values']['types']['all']) 
    && $form_state['values']['types']['all'] === 0) {
    foreach ($form_state['values']['types'] as $type => $value) {
      if ($type && $value !== 0) {
        $types[] = $type;
      }
    }
  }
  drupal_set_message('created: ' . var_export($form_state['values']['created'], TRUE));
  
  // Locations
//  if (!in_array('all', $form_state['values']['locations'])) {
//    foreach ($form_state['values']['types'] as $location) {
//      if ($location) {
//        $locations[] = $location;
//      }
//    }
//  }

  // Created
  $created['from'] = strtotime($form_state['values']['created']['from']['year']
    . '-' . $form_state['values']['created']['from']['month']
    . '-' . $form_state['values']['created']['from']['day']);
  $created['to'] = strtotime($form_state['values']['created']['to']['year']
    . '-' . $form_state['values']['created']['to']['month']
    . '-' . $form_state['values']['created']['to']['day'] . ' 23:59:59');

  // Updated
  $updated['from'] = strtotime($form_state['values']['updated']['from']['year']
    . '-' . $form_state['values']['updated']['from']['month']
    . '-' . $form_state['values']['updated']['from']['day']);
  $updated['to'] = strtotime($form_state['values']['updated']['to']['year']
    . '-' . $form_state['values']['updated']['to']['month']
    . '-' . $form_state['values']['updated']['to']['day'] . ' 23:59:59');

  $advanced = $form_state['values']['advanced'];
  $source = NULL;

  $batch = array(
    'operations' => array(
      array('xc_metadata_delete_batch', array($types, $locations, $created, 
        $updated, $advanced, $source)),
    ),
    'finished' => 'xc_metadata_delete_batch_finished',
    'title' => t('Metadata deletion in progress...'),
    'init_message' => t('Beginning metadata deletion process...'),
    'progress_message' => t('Deleting nodes') . " | " 
      . l('Stop process', 'xc/metadata/delete'),
    'error_message' => t('Metadata deletion has encountered an error.'),
  );
  batch_set($batch);
}

/**
 * Metadata deletion batch API operation
 * @param $types (Array) 
 *   The array of metadata types to delete
 * @param $locations (Array) 
 *   The array of location identifiers
 * @param $created (Array) 
 *   The timestaps of the date range of creation. The array has two keys: from 
 *   and to. Both contains a timestamp.
 * @param $updated (Array) 
 *   The timestaps of the date range of update. The array has two keys: 
 *   from and to. Both contains a timestamp.
 * @param $advanced
 * @param $source_id (int) 
 *   The source id of the XCEntity
 * @param $context
 * @return unknown_type
 */
function xc_metadata_delete_batch($types, $locations, $created, $updated, 
    $advanced, $source_id, &$context) {
  xc_log_info('delete', var_export($types, TRUE) . ', ' . var_export($locations, TRUE) . ', '
    . var_export($created, TRUE) . ', ' . var_export($updated, TRUE) . ', '
    . var_export($advanced, TRUE));
  // Begin the correct SQL for counting and selecting
  $count_sql  = "SELECT COUNT(DISTINCT metadata_id) ";
  $select_sql = "SELECT node_id ";
  $date_operation = isset($advanced['date_operation']) && 
    in_array($advanced['date_operation'], array('AND', 'OR')) 
    ? $advanced['date_operation'] 
    : 'AND';

  // SQL that is used by both
  $_sql = ' FROM {xc_entity_properties} '
        . ' WHERE ((created BETWEEN \'' . $created['from'] . '\' AND \'' 
        . $created['to'] . '\') ' . $date_operation . ' (updated BETWEEN \'' 
        . $updated['from'] . '\' AND \'' . $updated['to'] . '\'))';
  if ($types) {
    $_sql .= ' AND metadata_type IN (\'' . implode("','", $types) . '\') ';
  }
  if (!empty($source_id)) {
    $_sql .= ' AND source_id = ' . $source_id;
  }
  //  if ($locations) {
//    $_sql .= " AND type IN (". implode(',', $locations) .") ";
//  }

  // Combine SQL
  $count_sql  .= $_sql;
  $select_sql .= $_sql;

  // Sandbox and context
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current'] = 0;
    $context['sandbox']['max'] = db_result(db_query($count_sql));
  }

  $limit = !empty($advanced['batch_limit']) ? $advanced['batch_limit'] : 200;

  $result = db_query_range($select_sql, 0, 0, $limit);
  while ($entity = db_fetch_object($result)) {
    node_delete($entity->node_id);

    // Store some result for post-processing in the finished callback.
    $context['results'][] = $entity->node_id;

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['sandbox']['current'] = $entity->node_id;
    $context['message'] = t('Now deleting  %progress out of %max entities/nodes', array('%progress' => $context['sandbox']['progress'], '%max' => $context['sandbox']['max']));
  }

  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }

  // Drupal dumps too much messages into the session
  unset($_SESSION['messages']);
}

/**
 * Metadata deletion batch API finished callback
 */
function xc_metadata_delete_batch_finished($success, $results, $operations) {
  if ($success) {
    // Run commit and optimize incase it's needed
    _xc_commit();
    _xc_optimize();
    
    $message = t('Successfully deleted %count metadata entity/nodes.', 
      array('%count' => count($results)));
  }
  else {
    $error_operation = reset($operations);
    $message = t('An error occurred while deleting %error_operation with '
      . 'arguments: @arguments', array('%error_operation' => $error_operation[0], 
      '@arguments' => print_r($error_operation[1], TRUE)));
  }
  drupal_set_message($message);
  drupal_goto('xc/metadata/delete');
}

/**
 * Delete all records belonging to a given source and created of updated 
 * between 'from' and 'to' timestamps.
 * 
 * @param $source_id (int)
 *   The source id
 * @param $from (timestamp)
 *   The starting point of the date range
 * @param $to (timestamp)
 *   The ending point of the date range
 * @return unknown_type
 */
function xc_metadata_delete_by_source($source_id, $from, $to) {
  $created = $updated = array(
    'from' => $from,
    'to'   => $to
  );
  xc_log_info('delete', var_export($created, TRUE));
  xc_log_info('delete', var_export($updated, TRUE));
  
  $batch = array(
    'operations' => array(
      array('xc_metadata_delete_batch', array(NULL, NULL, $created, $updated, 
        array('date_operation' => 'OR'), $source_id)),
    ),
    'finished' => 'xc_metadata_delete_batch_finished',
    'title' => t('Metadata deletion in progress...'),
    'init_message' => t('Beginning metadata deletion process...'),
    'progress_message' => t('Deleting nodes') . " | " 
      . l('Stop process', 'xc/metadata/delete'),
    'error_message' => t('Metadata deletion has encountered an error.'),
  );
  batch_set($batch);
  batch_process('xc/metadata/delete');
}